@@ -31,7 +31,6 @@
 #include "gsimpleasyncresult.h"
 #include "gioerror.h"
 
-#include "gioalias.h"
 
 /**
  * SECTION:ginputstream
@@ -48,7 +47,7 @@
  * All of these functions have async variants too.
  **/
 
-G_DEFINE_TYPE (GInputStream, g_input_stream, G_TYPE_OBJECT);
+G_DEFINE_ABSTRACT_TYPE (GInputStream, g_input_stream, G_TYPE_OBJECT);
 
 struct _GInputStreamPrivate {
   guint closed : 1;
@@ -561,11 +560,10 @@ g_input_stream_read_async (GInputStream        *stream,
 
   if (!g_input_stream_set_pending (stream, &error))
     {
-      g_simple_async_report_gerror_in_idle (G_OBJECT (stream),
+      g_simple_async_report_take_gerror_in_idle (G_OBJECT (stream),
 					    callback,
 					    user_data,
 					    error);
-      g_error_free (error);
       return;
     }
 
@@ -617,35 +615,35 @@ g_input_stream_read_finish (GInputStream  *stream,
  * g_input_stream_skip_async:
  * @stream: A #GInputStream.
  * @count: the number of bytes that will be skipped from the stream
- * @io_priority: the <link linkend="io-priority">I/O priority</link> 
- * of the request. 
- * @cancellable: optional #GCancellable object, %NULL to ignore. 
+ * @io_priority: the <link linkend="io-priority">I/O priority</link>
+ * of the request.
+ * @cancellable: optional #GCancellable object, %NULL to ignore.
  * @callback: callback to call when the request is satisfied
  * @user_data: the data to pass to callback function
  *
  * Request an asynchronous skip of @count bytes from the stream.
- * When the operation is finished @callback will be called. 
- * You can then call g_input_stream_skip_finish() to get the result of the 
- * operation.
+ * When the operation is finished @callback will be called.
+ * You can then call g_input_stream_skip_finish() to get the result
+ * of the operation.
  *
- * During an async request no other sync and async calls are allowed, and will
- * result in %G_IO_ERROR_PENDING errors. 
+ * During an async request no other sync and async calls are allowed,
+ * and will result in %G_IO_ERROR_PENDING errors.
  *
  * A value of @count larger than %G_MAXSSIZE will cause a %G_IO_ERROR_INVALID_ARGUMENT error.
  *
- * On success, the number of bytes skipped will be passed to the
- * callback. It is not an error if this is not the same as the requested size, as it
+ * On success, the number of bytes skipped will be passed to the callback.
+ * It is not an error if this is not the same as the requested size, as it
  * can happen e.g. near the end of a file, but generally we try to skip
  * as many bytes as requested. Zero is returned on end of file
  * (or if @count is zero), but never otherwise.
  *
- * Any outstanding i/o request with higher priority (lower numerical value) will
- * be executed before an outstanding request with lower priority. Default
- * priority is %G_PRIORITY_DEFAULT.
+ * Any outstanding i/o request with higher priority (lower numerical value)
+ * will be executed before an outstanding request with lower priority.
+ * Default priority is %G_PRIORITY_DEFAULT.
  *
- * The asyncronous methods have a default fallback that uses threads to implement
- * asynchronicity, so they are optional for inheriting classes. However, if you
- * override one you must override all.
+ * The asynchronous methods have a default fallback that uses threads to
+ * implement asynchronicity, so they are optional for inheriting classes.
+ * However, if you override one, you must override all.
  **/
 void
 g_input_stream_skip_async (GInputStream        *stream,
@@ -686,11 +684,10 @@ g_input_stream_skip_async (GInputStream        *stream,
 
   if (!g_input_stream_set_pending (stream, &error))
     {
-      g_simple_async_report_gerror_in_idle (G_OBJECT (stream),
+      g_simple_async_report_take_gerror_in_idle (G_OBJECT (stream),
 					    callback,
 					    user_data,
 					    error);
-      g_error_free (error);
       return;
     }
 
@@ -785,11 +782,10 @@ g_input_stream_close_async (GInputStream        *stream,
 
   if (!g_input_stream_set_pending (stream, &error))
     {
-      g_simple_async_report_gerror_in_idle (G_OBJECT (stream),
+      g_simple_async_report_take_gerror_in_idle (G_OBJECT (stream),
 					    callback,
 					    user_data,
 					    error);
-      g_error_free (error);
       return;
     }
   
@@ -948,10 +944,7 @@ read_async_thread (GSimpleAsyncResult *res,
 				   op->buffer, op->count_requested,
 				   cancellable, &error);
   if (op->count_read == -1)
-    {
-      g_simple_async_result_set_from_error (res, error);
-      g_error_free (error);
-    }
+    g_simple_async_result_take_error (res, error);
 }
 
 static void
@@ -1013,10 +1006,7 @@ skip_async_thread (GSimpleAsyncResult *res,
 				   op->count_requested,
 				   cancellable, &error);
   if (op->count_skipped == -1)
-    {
-      g_simple_async_result_set_from_error (res, error);
-      g_error_free (error);
-    }
+    g_simple_async_result_take_error (res, error);
 }
 
 typedef struct {
@@ -1067,13 +1057,12 @@ skip_callback_wrapper (GObject      *source_object,
 
   if (ret == -1)
     {
-      if (data->count_skipped && 
-	  error->domain == G_IO_ERROR &&
-	  error->code == G_IO_ERROR_CANCELLED)
-	{ /* No error, return partial read */ }
+      if (data->count_skipped &&
+          g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED))
+	/* No error, return partial read */
+	g_error_free (error);
       else
-	g_simple_async_result_set_from_error (simple, error);
-      g_error_free (error);
+	g_simple_async_result_take_error (simple, error);
     }
 
   /* Complete immediately, not in idle, since we're already in a mainloop callout */
@@ -1165,10 +1154,7 @@ close_async_thread (GSimpleAsyncResult *res,
     {
       result = class->close_fn (G_INPUT_STREAM (object), cancellable, &error);
       if (!result)
-	{
-	  g_simple_async_result_set_from_error (res, error);
-	  g_error_free (error);
-	}
+        g_simple_async_result_take_error (res, error);
     }
 }
 
@@ -1204,6 +1190,3 @@ g_input_stream_real_close_finish (GInputStream  *stream,
   g_warn_if_fail (g_simple_async_result_get_source_tag (simple) == g_input_stream_real_close_async);
   return TRUE;
 }
-
-#define __G_INPUT_STREAM_C__
-#include "gioaliasdef.c"
