@@ -209,13 +209,13 @@ G_GNUC_INTERNAL void _gio_marshal_INT__OBJECT (GClosure     *closure,
                                                gpointer      invocation_hint,
                                                gpointer      marshal_data);
 
-/* VOID:UINT64 (./gio-marshal.list:27) */
-G_GNUC_INTERNAL void _gio_marshal_VOID__UINT64 (GClosure     *closure,
-                                                GValue       *return_value,
-                                                guint         n_param_values,
-                                                const GValue *param_values,
-                                                gpointer      invocation_hint,
-                                                gpointer      marshal_data);
+/* VOID:INT64 (./gio-marshal.list:27) */
+G_GNUC_INTERNAL void _gio_marshal_VOID__INT64 (GClosure     *closure,
+                                               GValue       *return_value,
+                                               guint         n_param_values,
+                                               const GValue *param_values,
+                                               gpointer      invocation_hint,
+                                               gpointer      marshal_data);
 
 G_END_DECLS
 
