@@ -43,6 +43,44 @@
  * while the clock is not running, in which case the rate of repairs
  * will be rate limited as if the clock were running.
  *
+ * #GPeriodic has a configurable priority range consisting of a high and
+ * low value.  Other sources with a priority higher than the high value
+ * might starve #GPeriodic and sources with the priority lower than the
+ * low value may be starved by #GPeriodic.
+ *
+ * #GPeriodic will engage in dynamic scheduling with respect to sources
+ * that have their priorities within the high to low range.  A given
+ * #GPeriodic will ensure that the events dispatched from itself are
+ * generally using less than 50% of the CPU (on average) if other tasks
+ * are pending.  If no other sources within the range are pending then
+ * #GPeriodic will use up to all of the available CPU (which can lead to
+ * starvation of lower-priority sources, as mentioned above).  The 50%
+ * figure is entirely arbitrary and may change or become configurable in
+ * the future.
+ *
+ * For example, if a #GPeriodic has been set to run at 10Hz and a
+ * particular iteration uses 140ms of time, then 2 ticks will be
+ * "skipped" to give other sources a chance to run (ie: the next tick
+ * will occur 300ms later rather than 100ms later, giving 160ms of time
+ * for other sources).
+ *
+ * This means that the high priority value for #GPeriodic should be set
+ * quite high (above anything else) and the low priority value for
+ * #GPeriodic should be set lower than everything except true "idle"
+ * handlers (ie: things that you want to run only when the program is
+ * truly idle).
+ *
+ * #GPeriodic generally assumes that although the things attached to it
+ * may be poorly behaved in terms of non-yielding behaviour (either
+ * individually or in aggregate), the other sources on the main loop
+ * should be "well behaved".  Other sources should try not to block the
+ * CPU for a substantial portion of the periodic interval.
+ *
+ * The sources attached to a #GPeriodic are permitted to be somewhat
+ * less well-behaved because they are generally rendering the UI for the
+ * user (which should be done smoothly) and also because they will be
+ * throttled by #GPeriodic.
+ *
  * #GPeriodic is intended to be used as a paint clock for managing
  * geometry updates and painting of windows.
  *
@@ -57,8 +95,6 @@
  * Since: 2.28
  **/
 
-typedef GObjectClass GPeriodicClass;
-
 typedef struct
 {
   GPeriodicTickFunc callback;
@@ -67,29 +103,25 @@ typedef struct
   guint             id;
 } GPeriodicTick;
 
-typedef struct
-{
-  GPeriodicRepairFunc callback;
-  gpointer            user_data;
-  GDestroyNotify      notify;
-} GPeriodicRepair;
-
 typedef struct
 {
   GSource    source;
   GPeriodic *periodic;
 } GPeriodicSource;
 
-struct _GPeriodic
+struct _GPeriodicPrivate
 {
-  GObject  parent_instance;
   GSource *source;            /* GPeriodicSource */
   guint64  last_run;
   guint    blocked;
   guint    hz;
+  guint    skip_frames;
+  guint    stop_skip_id;
+  gint     low_priority;
 
   GSList  *ticks;             /* List<GPeriodicTick> */
-  GSList  *repairs;           /* List<GPeriodicRepair> */
+
+  guint    damaged   : 1;
 
   /* debug */
   guint    in_tick   : 1;
@@ -104,7 +136,8 @@ enum
 {
   PROP_NONE,
   PROP_HZ,
-  PROP_PRIORITY
+  PROP_HIGH_PRIORITY,
+  PROP_LOW_PRIORITY
 };
 
 static guint g_periodic_tick;
@@ -113,62 +146,109 @@ static guint g_periodic_repair;
 static guint64
 g_periodic_get_microticks (GPeriodic *periodic)
 {
-  guint64 microticks;
-  GTimeSpec timespec;
+  return g_source_get_time (periodic->priv->source) * periodic->priv->hz;
+}
 
-  g_source_get_time (periodic->source, &timespec);
+static gboolean
+g_periodic_stop_skip (gpointer data)
+{
+  GPeriodic *periodic = data;
 
-  microticks = timespec.tv_sec;
-  microticks *= 1000000;
-  microticks += timespec.tv_nsec / 1000;
-  microticks *= periodic->hz;
+  periodic->priv->skip_frames = 0;
 
-  return microticks;
+  g_message ("Skipping frames ends");
+
+  periodic->priv->stop_skip_id = 0;
+
+  return FALSE;
+}
+
+static void
+g_periodic_real_tick (GPeriodic *periodic,
+                      gint64     timestamp)
+{
+  GSList *iter;
+
+  for (iter = periodic->priv->ticks; iter; iter = iter->next)
+    {
+      GPeriodicTick *tick = iter->data;
+
+      tick->callback (periodic, timestamp, tick->user_data);
+    }
+}
+
+static void
+g_periodic_real_repair (GPeriodic *periodic)
+{
+  periodic->priv->damaged = FALSE;
 }
 
 static void
 g_periodic_run (GPeriodic *periodic)
 {
-  g_assert (periodic->blocked == 0);
+  gint64 start, usec;
 
-  if (periodic->ticks)
+  g_assert (periodic->priv->blocked == 0);
+
+  start = g_get_monotonic_time ();
+
+  if (periodic->priv->ticks)
     {
       guint64 microseconds;
-      GSList *iter;
 
-      microseconds = periodic->last_run / periodic->hz;
-
-      periodic->in_tick = TRUE;
-      for (iter = periodic->ticks; iter; iter = iter->next)
-        {
-          GPeriodicTick *tick = iter->data;
-
-          tick->callback (periodic, microseconds, tick->user_data);
-        }
+      periodic->priv->in_tick = TRUE;
+      microseconds = periodic->priv->last_run / periodic->priv->hz;
       g_signal_emit (periodic, g_periodic_tick, 0, microseconds);
-      periodic->in_tick = FALSE;
+      periodic->priv->in_tick = FALSE;
     }
 
-  g_assert (periodic->blocked == 0);
+  g_assert (periodic->priv->blocked == 0);
 
-  if (periodic->repairs)
+  if (periodic->priv->damaged)
     {
-      periodic->in_repair = TRUE;
-      while (periodic->repairs)
-        {
-          GPeriodicRepair *repair = periodic->repairs->data;
-
-          repair->callback (periodic, repair->user_data);
-
-          periodic->repairs = g_slist_remove (periodic->repairs, repair);
-
-          if (repair->notify)
-            repair->notify (repair->user_data);
-
-          g_slice_free (GPeriodicRepair, repair);
-        }
+      periodic->priv->in_repair = TRUE;
       g_signal_emit (periodic, g_periodic_repair, 0);
-      periodic->in_repair = FALSE;
+      periodic->priv->in_repair = FALSE;
+    }
+
+  g_assert (!periodic->priv->damaged);
+
+  usec = g_get_monotonic_time () - start;
+  g_assert (usec >= 0);
+
+  /* Take the time it took to render, multiply by two and round down to
+   * a whole number of frames.  This ensures that we don't take more
+   * than 50% of the CPU with rendering.
+   *
+   * Examples (at 10fps for easy math.  1 frame = 100ms):
+   *
+   *   0-49ms to render: no frames skipped
+   *
+   *     We used less than half of the time to render.  OK.  We will run
+   *     the next frame 100ms after this one ran (no skips).
+   *
+   *   50-99ms to render: 1 frame skipped
+   *
+   *     We used more than half the time.  Skip one frame so that we run
+   *     200ms later rather than 100ms later.  We already used up to
+   *     99ms worth of that 200ms window, so that gives 101ms for other
+   *     things to run (50%).  For figures closer to 50ms the other
+   *     stuff will actually get more than 50%.
+   *
+   *   100-150ms to render: 2 frames skipped, etc.
+   */
+  periodic->priv->skip_frames = 2 * usec * periodic->priv->hz / 1000000;
+
+  if (periodic->priv->skip_frames)
+    {
+      g_message ("Slow painting; (possibly) skipping %d frames\n",
+                 periodic->priv->skip_frames);
+
+      if (periodic->priv->stop_skip_id == 0)
+        periodic->priv->stop_skip_id =
+          g_idle_add_full (periodic->priv->low_priority,
+                           g_periodic_stop_skip,
+                           periodic, NULL);
     }
 }
 
@@ -178,12 +258,12 @@ g_periodic_prepare (GSource *source,
 {
   GPeriodic *periodic = PERIODIC_FROM_SOURCE (source);
 
-  if ((periodic->ticks || periodic->repairs) && !periodic->blocked)
+  if ((periodic->priv->ticks || periodic->priv->damaged) && !periodic->priv->blocked)
     /* We need to run. */
     {
       gint64 remaining;
      
-      remaining = periodic->last_run + 1000000 -
+      remaining = periodic->priv->last_run + 1000000 * (1 + periodic->priv->skip_frames) -
                   g_periodic_get_microticks (periodic);
 
       if (remaining > 0)
@@ -195,7 +275,7 @@ g_periodic_prepare (GSource *source,
            * that not enough time has passed and want to sleep again, so
            * save ourselves the future bother.
            */
-          *timeout = (remaining + periodic->hz - 1) / periodic->hz / 1000;
+          *timeout = (remaining + periodic->priv->hz - 1) / periodic->priv->hz / 1000;
 
           return FALSE;
         }
@@ -220,13 +300,13 @@ g_periodic_check (GSource *source)
 {
   GPeriodic *periodic = PERIODIC_FROM_SOURCE (source);
 
-  if ((periodic->ticks || periodic->repairs) && !periodic->blocked)
+  if ((periodic->priv->ticks || periodic->priv->damaged) && !periodic->priv->blocked)
     /* We need to run. */
     {
       guint64 now = g_periodic_get_microticks (periodic);
 
       /* Run if it's not too soon. */
-      return !(now < periodic->last_run + 1000000);
+      return !(now < periodic->priv->last_run + 1000000 * (periodic->priv->skip_frames + 1));
     }
 
   else
@@ -243,7 +323,7 @@ g_periodic_dispatch (GSource     *source,
   guint64 elapsed_ticks;
   guint64 now;
 
-  g_assert (periodic->blocked == 0);
+  g_assert (periodic->priv->blocked == 0);
 
   /* Update the last_run.
    *
@@ -260,13 +340,13 @@ g_periodic_dispatch (GSource     *source,
    *   - resets our stride if we missed a frame
    */
   now = g_periodic_get_microticks (periodic);
-  elapsed_ticks = (now - periodic->last_run) / 1000000;
+  elapsed_ticks = (now - periodic->priv->last_run) / 1000000;
   g_assert (elapsed_ticks > 0);
 
   if G_LIKELY (elapsed_ticks == 1)
-    periodic->last_run += 1000000;
+    periodic->priv->last_run += 1000000;
   else
-    periodic->last_run = now;
+    periodic->priv->last_run = now;
 
   g_periodic_run (periodic);
 
@@ -281,12 +361,16 @@ g_periodic_get_property (GObject *object, guint prop_id,
 
   switch (prop_id)
     {
-    case PROP_PRIORITY:
-      g_value_set_int (value, g_source_get_priority (periodic->source));
+    case PROP_HIGH_PRIORITY:
+      g_value_set_int (value, g_source_get_priority (periodic->priv->source));
+      break;
+
+    case PROP_LOW_PRIORITY:
+      g_value_set_int (value, periodic->priv->low_priority);
       break;
 
     case PROP_HZ:
-      g_value_set_uint (value, periodic->hz);
+      g_value_set_uint (value, periodic->priv->hz);
       break;
 
     default:
@@ -302,12 +386,16 @@ g_periodic_set_property (GObject *object, guint prop_id,
 
   switch (prop_id)
     {
-    case PROP_PRIORITY:
-      g_source_set_priority (periodic->source, g_value_get_int (value));
+    case PROP_HIGH_PRIORITY:
+      g_source_set_priority (periodic->priv->source, g_value_get_int (value));
+      break;
+
+    case PROP_LOW_PRIORITY:
+      periodic->priv->low_priority = g_value_get_int (value);
       break;
 
     case PROP_HZ:
-      periodic->hz = g_value_get_uint (value);
+      periodic->priv->hz = g_value_get_uint (value);
       break;
 
     default:
@@ -320,8 +408,8 @@ g_periodic_finalize (GObject *object)
 {
   GPeriodic *periodic = G_PERIODIC (object);
 
-  g_source_destroy (periodic->source);
-  g_source_unref (periodic->source);
+  g_source_destroy (periodic->priv->source);
+  g_source_unref (periodic->priv->source);
 
   G_OBJECT_CLASS (g_periodic_parent_class)
     ->finalize (object);
@@ -334,38 +422,59 @@ g_periodic_init (GPeriodic *periodic)
     g_periodic_prepare, g_periodic_check, g_periodic_dispatch
   };
 
-  periodic->source = g_source_new (&source_funcs, sizeof (GPeriodicSource));
-  ((GPeriodicSource *) periodic->source)->periodic = periodic;
-  g_source_attach (periodic->source, g_main_context_get_thread_default ());
+  periodic->priv = G_TYPE_INSTANCE_GET_PRIVATE (periodic,
+                                                G_TYPE_PERIODIC,
+                                                GPeriodicPrivate);
+
+  periodic->priv->source = g_source_new (&source_funcs,
+                                         sizeof (GPeriodicSource));
+  ((GPeriodicSource *) periodic->priv->source)->periodic = periodic;
+  g_source_attach (periodic->priv->source,
+                   g_main_context_get_thread_default ());
 }
 
 static void
-g_periodic_class_init (GObjectClass *class)
+g_periodic_class_init (GPeriodicClass *class)
 {
-  class->get_property = g_periodic_get_property;
-  class->set_property = g_periodic_set_property;
-  class->finalize = g_periodic_finalize;
+  GObjectClass *object_class = G_OBJECT_CLASS (class);
+
+  class->tick = g_periodic_real_tick;
+  class->repair = g_periodic_real_repair;
+
+  object_class->get_property = g_periodic_get_property;
+  object_class->set_property = g_periodic_set_property;
+  object_class->finalize = g_periodic_finalize;
 
   g_periodic_tick = g_signal_new ("tick", G_TYPE_PERIODIC,
-                                  G_SIGNAL_RUN_LAST, 0, NULL, NULL,
-                                  _gio_marshal_VOID__UINT64,
-                                  G_TYPE_NONE, 1, G_TYPE_UINT64);
+                                  G_SIGNAL_RUN_LAST,
+                                  G_STRUCT_OFFSET(GPeriodicClass, tick),
+                                  NULL, NULL, _gio_marshal_VOID__INT64,
+                                  G_TYPE_NONE, 1, G_TYPE_INT64);
   g_periodic_repair = g_signal_new ("repair", G_TYPE_PERIODIC,
-                                    G_SIGNAL_RUN_LAST, 0, NULL, NULL,
-                                    g_cclosure_marshal_VOID__VOID,
+                                    G_SIGNAL_RUN_LAST,
+                                    G_STRUCT_OFFSET (GPeriodicClass, repair),
+                                    NULL, NULL, g_cclosure_marshal_VOID__VOID,
                                     G_TYPE_NONE, 0);
 
-  g_object_class_install_property (class, PROP_HZ,
+  g_object_class_install_property (object_class, PROP_HZ,
     g_param_spec_uint ("hz", "ticks per second",
                        "rate (in Hz) at which the 'tick' signal is emitted",
                        1, 120, 1, G_PARAM_READWRITE |
                        G_PARAM_CONSTRUCT_ONLY | G_PARAM_STATIC_STRINGS));
 
-  g_object_class_install_property (class, PROP_PRIORITY,
-    g_param_spec_int ("priority", "priority level",
+  g_object_class_install_property (object_class, PROP_HIGH_PRIORITY,
+    g_param_spec_int ("high-priority", "high priority level",
                       "the GSource priority level to run at",
                       G_MININT, G_MAXINT, 0, G_PARAM_READWRITE |
                       G_PARAM_CONSTRUCT_ONLY | G_PARAM_STATIC_STRINGS));
+
+  g_object_class_install_property (object_class, PROP_LOW_PRIORITY,
+    g_param_spec_int ("low-priority", "low priority level",
+                      "ignore tasks below this priority level",
+                      G_MININT, G_MAXINT, 0, G_PARAM_READWRITE |
+                      G_PARAM_CONSTRUCT_ONLY | G_PARAM_STATIC_STRINGS));
+
+  g_type_class_add_private (class, sizeof (GPeriodicPrivate));
 }
 
 /**
@@ -389,15 +498,15 @@ void
 g_periodic_block (GPeriodic *periodic)
 {
   g_return_if_fail (G_IS_PERIODIC (periodic));
-  g_return_if_fail (periodic->in_repair);
+  g_return_if_fail (periodic->priv->in_repair);
 
-  periodic->blocked++;
+  periodic->priv->blocked++;
 }
 
 /**
  * g_periodic_unblock:
  * @periodic: a #GPeriodic clock
- * @unblock_time: the unblock time, or %NULL
+ * @unblock_time: the unblock time
  *
  * Reverses the effect of a previous call to g_periodic_block().
  *
@@ -406,8 +515,7 @@ g_periodic_block (GPeriodic *periodic)
  * damaged.
  *
  * @unblock_time is the monotonic time, as per g_get_monotonic_time(),
- * at which the event causing the unblock occured.  If it is %NULL then
- * g_get_monotonic_time() is called internally.
+ * at which the event causing the unblock occured.
  *
  * This function may not be called from handlers of any signal emitted
  * by @periodic.
@@ -416,28 +524,16 @@ g_periodic_block (GPeriodic *periodic)
  **/
 void
 g_periodic_unblock (GPeriodic       *periodic,
-                    const GTimeSpec *unblock_time)
+                    gint64           unblock_time)
 {
   g_return_if_fail (G_IS_PERIODIC (periodic));
-  g_return_if_fail (!periodic->in_repair);
-  g_return_if_fail (!periodic->in_tick);
-  g_return_if_fail (periodic->blocked);
+  g_return_if_fail (!periodic->priv->in_repair);
+  g_return_if_fail (!periodic->priv->in_tick);
+  g_return_if_fail (periodic->priv->blocked);
 
-  if (--periodic->blocked)
+  if (--periodic->priv->blocked)
     {
-      GTimeSpec now;
-
-      if (unblock_time == NULL)
-        {
-          g_get_monotonic_time (&now);
-          unblock_time = &now;
-        }
-
-      periodic->last_run = unblock_time->tv_sec;
-      periodic->last_run *= 1000000;
-      periodic->last_run += unblock_time->tv_nsec / 1000;
-      periodic->last_run *= periodic->hz;
-
+      periodic->priv->last_run = unblock_time * periodic->priv->hz;
       g_periodic_run (periodic);
     }
 }
@@ -487,7 +583,7 @@ g_periodic_add (GPeriodic         *periodic,
   static guint id;
 
   g_return_val_if_fail (G_IS_PERIODIC (periodic), 0);
-  g_return_val_if_fail (!periodic->in_repair, 0);
+  g_return_val_if_fail (!periodic->priv->in_repair, 0);
 
   tick = g_slice_new (GPeriodicTick);
   tick->callback = callback;
@@ -495,7 +591,7 @@ g_periodic_add (GPeriodic         *periodic,
   tick->notify = notify;
   tick->id = ++id;
 
-  periodic->ticks = g_slist_prepend (periodic->ticks, tick);
+  periodic->priv->ticks = g_slist_prepend (periodic->priv->ticks, tick);
 
   return tick->id;
 }
@@ -522,9 +618,9 @@ g_periodic_remove (GPeriodic *periodic,
   GSList **iter;
 
   g_return_if_fail (G_IS_PERIODIC (periodic));
-  g_return_if_fail (!periodic->in_repair);
+  g_return_if_fail (!periodic->priv->in_repair);
 
-  for (iter = &periodic->ticks; *iter; iter = &(*iter)->next)
+  for (iter = &periodic->priv->ticks; *iter; iter = &(*iter)->next)
     {
       GPeriodicTick *tick = (*iter)->data;
 
@@ -547,45 +643,21 @@ g_periodic_remove (GPeriodic *periodic,
 /**
  * g_periodic_damaged:
  * @periodic: a #GPeriodic clock
- * @callback: a #GPeriodicRepairFunc
- * @user_data: data for @callback
- * @notify: for freeing @user_data when it is no longer needed
  *
- * Report damage and schedule @callback to be called during the next
- * repair phase.  Multiple registrations result in multiple invocations,
- * so you should track for yourself if you are already registered.
+ * Report damage and schedule the "repair" signal to be emitted during
+ * the next repair phase.
  *
  * You may not call this function during the repair phase.
  *
  * Since: 2.28
  **/
-/**
- * GPeriodicRepairFunc:
- * @periodic: the #GPeriodic clock that the damage was reported to
- * @user_data: the user data given to g_periodic_damaged()
- *
- * The signature of the callback function that is called during the
- * repair phase when damaged was reported using g_periodic_damaged().
- *
- * Since: 2.28
- **/
 void
-g_periodic_damaged (GPeriodic           *periodic,
-                    GPeriodicRepairFunc  callback,
-                    gpointer             user_data,
-                    GDestroyNotify       notify)
+g_periodic_damaged (GPeriodic *periodic)
 {
-  GPeriodicRepair *repair;
-
   g_return_if_fail (G_IS_PERIODIC (periodic));
-  g_return_if_fail (!periodic->in_repair);
+  g_return_if_fail (!periodic->priv->in_repair);
 
-  repair = g_slice_new (GPeriodicRepair);
-  repair->callback = callback;
-  repair->user_data = user_data;
-  repair->notify = notify;
-
-  periodic->repairs = g_slist_prepend (periodic->repairs, repair);
+  periodic->priv->damaged = TRUE;
 }
 
 /**
@@ -601,23 +673,41 @@ g_periodic_damaged (GPeriodic           *periodic,
 guint
 g_periodic_get_hz (GPeriodic *periodic)
 {
-  return periodic->hz;
+  return periodic->priv->hz;
 }
 
 /**
- * g_periodic_get_priority:
+ * g_periodic_get_high_priority:
  * @periodic: a #GPeriodic clock
  *
  * Gets the #GSource priority of the clock.
  *
- * Returns: the priority level
+ * Returns: the high priority level
  *
  * Since: 2.28
  **/
 gint
-g_periodic_get_priority (GPeriodic *periodic)
+g_periodic_get_high_priority (GPeriodic *periodic)
 {
-  return g_source_get_priority (periodic->source);
+  return g_source_get_priority (periodic->priv->source);
+}
+
+/**
+ * g_periodic_get_low_priority:
+ * @periodic: a #GPeriodic clock
+ *
+ * Gets the priority level that #GPeriodic uses to check for mainloop
+ * inactivity.  Other sources scheduled below this level of priority are
+ * effectively ignored by #GPeriodic and may be starved.
+ *
+ * Returns: the low priority level
+ *
+ * Since: 2.28
+ **/
+gint
+g_periodic_get_low_priority (GPeriodic *periodic)
+{
+  return periodic->priv->low_priority;
 }
 
 /**
@@ -641,12 +731,14 @@ g_periodic_get_priority (GPeriodic *periodic)
  **/
 GPeriodic *
 g_periodic_new (guint hz,
-                gint  priority)
+                gint  high_priority,
+                gint  low_priority)
 {
   g_return_val_if_fail (1 <= hz && hz <= 120, NULL);
 
   return g_object_new (G_TYPE_PERIODIC,
                        "hz", hz,
-                       "priority", priority,
+                       "high-priority", high_priority,
+                       "low-priority", low_priority,
                        NULL);
 }
