@@ -24,12 +24,12 @@
 
 #include <string.h>
 
-#include "glib.h"
-
-#ifdef _
-#warning "FIXME remove gettext hack"
-#endif
+#include "gshell.h"
 
+#include "gslist.h"
+#include "gstrfuncs.h"
+#include "gstring.h"
+#include "gtestutils.h"
 #include "glibintl.h"
 
 /**
